#! /usr/bin/perl
#
#                          LaTeX2pic
#
#                          by Ivan Andrus
#                     E-mail: gvol@math.byu.edu
#               http://math.byu.edu/~gvol/winedt.html
                   $version = '1.2 (2004-10-1)';
#
#
#   Write your equation (or anything else you can typeset with LaTeX)
#   in a file like:
#
#       \documentclass[12pt]{article}
#       \pagestyle{empty}
#       \begin{document}
#
#       \begin{displaymath}
#       \bf  % Compiled formulae often look better in boldface
#       \int H(x,x')\psi(x')dx' = -\frac{\hbar2}{2m}\frac{d2}{dx2}
#                                 \psi(x)+V(x)\psi(x)
#       \end{displaymath}
#
#       \end{document}
#
#   The "\pagestyle{empty}" is required to avoid generating a huge
#   image with a page number at the bottom.
#
#   Then (assuming you have all the software described below installed
#   properly), you can simply say:
#
#       LaTeX2pic.pl [ options ] (texfile [-rename newname]) ...
#
#   to compile filename.tex to filename.gif, an interlaced,
#   transparent background GIF file ready to use an an inline image.
#   You can specify the base name, for example, "schrod", rather than
#   the full name of the TeX file ("schrod.tex").  TeX requires the
#   input file to have an extension of ".tex".  The command line
#   options are described in the help text at the end of this program
#   and in the "Default Configuration" section below.
#
#   Note that the -rename option comes after the .tex file which is to
#   be processed.  Thus you can specify many files to be run in batch mode,
#   and rename each of them separately.  Slightly non-standard, but I think
#   it works well enough.
#
#   A sample IMG tag, including the image width and height is printed
#   on standard error, for example:
#
#       <img src="schrod.gif" width="508" height="56">
#
#                         Required Software
#
#   This script requires the following software to be installed
#   in the standard manner.  Version numbers are those used in the
#   development and testing of the script.
#
#   Perl        5.8.0 (anything later than 4.036 should work)
#   TeX         3.14159 (Web2C 7.3.1)
#   LaTeX2e     <2000/06/01>
#   dvips       dvips 5.86
#   gswin32c    ??
#   ImageMagick ??
#
#   Default image creation modes
    $imageExt = 'jpg';
    $transparency = "-transparent white";
    $dpi = 150;
    $view = 0; #false

#   Command line option processing
    while ($ARGV[0] =~ m/^-/) {
        $_ = shift(@ARGV);
# Helps debug the MUI
# print $_ . "\n";
        s/^--/-/;                     # Allow GNU-style -- options
        if (m/^-trans\=(\d)/) {       # -trans={0,1} tranparent or not
            $transparency = "" if ($1 == 0);
            $transparency = "-transparent white" if ($1 == 1);
        } elsif (m/^-zoom\w*=(.*)/) {  # -mag=<num>
            $dpi*=$1/100 if $1>0;
        } elsif (m/^-h/) {            # -help
            &help();
            exit(0);
        } elsif (m/^-v/) {            # -version
            print("Version $version\n");
            exit(0);
        } elsif (m/^-imdisplay/) {    # -imdisplay
            $view=1;
        }else{                        # Otherwise assume its a file extension
            $imageExt = substr $_, 1;
       }
    }

#   Main file processing loop
    while(@ARGV) {
        $f = shift(@ARGV);
        $f =~ s/\.tex$//;
        &syscmd("echo x | latex $f \n");

        $cmd = "dvips -f $f -o $f.ps\n";
        &syscmd($cmd);

        &ps2jgeg();

        $cmd = "convert -trim -mattecolor white -frame 10x10 -interlace line jpeg:$f.jpeg $imageExt:$f.$imageExt\n";
        &syscmd($cmd);

    #   Rename if desired
        if($ARGV[0] =~ m/^-ren/){
            shift(@ARGV); #dump the rename
            if($ARGV[0] eq $f){
                shift(@ARGV)
            }else{
                &syscmd("move $f.$imageExt " . $ARGV[0] . ".$imageExt" );
                $f=shift(@ARGV);
            }
        }

    #   Collect size data and print the reference to include this figure,
    #   including width and height, to standard error.
        $r = `identify $f.$imageExt`;
        $r =~ m/(\d+)x(\d+)/;
        print(STDERR "<img src=\"$f.$imageExt\" width=\"$1\" height=\"$2\">\n");

        exec `imdisplay $f.$imageExt` if $view;
    }



    #   Echo and execute a system command
    sub syscmd {
        local ($cmd) = @_;
        print(STDERR "$cmd\n");
        system($cmd) == 0 || die("Error processing command:\n\t$cmd\n\t");
    }

    # change a ps 2 a jpeg via gswin32c
    sub ps2jgeg { # Converts postscript to png
        open BOB, "< $f.ps";
 Wh:    while($_=<BOB>){
            if($_=~ /%%BoundingBox: ([0-9]+) ([0-9]+) ([0-9]+) ([0-9]+)/m){
               $translate="-$1 -$2 translate";
                $width=int($dpi*($3-$1)/72);
                $height=int($dpi*($4-$2)/72);
                last Wh;
            }else{print ".";}
        }
        close BOB;
        print "\n";

        die "Your bounding box is messed up!" if ($width ==0);

        $cmd = "gswin32c -dNOPAUSE -dBATCH -dQUIET -sDEVICE=jpeg ".
               "-dPARANOIDSAFER -dTextAlphaBits=4 -dGraphicsAlphaBits=4 ".
               "-g${width}x$height -r$dpi -sOutputFile=$f.jpeg ".
               "-c $translate -f $f.ps";
        &syscmd($cmd);
    }

    #   Print help text
    sub help {
        print <<"EOD"
usage: LaTeX2pic [ options ] (texfile [-rename newname]) ...
    Options:
        -help           Print this message
        -version        Print version number
        -trans=1        Make transparent (default)
        -trans=0        Do not make transparent
        -imdisplay      Open the resulting image with imdisplay when done
        -zoom=<num>      Magnify the image by num% (default 100)
        -gif            Generate GIF image (default)
        -jpeg           Generate JPEG image
        -png            Generate PNG image
        -ext            Generate image or type "ext"
        -rename newname The newname desired (without the graphics extension)
For the latest version of this program
please visit the Web page:
   http://math.byu.edu/~gvol/winedt.html
EOD
;
    }
